/*================================================================
  File:     4_ComputeScalar.sql

  Summary:  Nested loop example with a large number of 
			random IOs on the inner loop

  SQL Server Versions: SQL Server 2012, 2014, 2016
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/


/*
	Compute Scalar can be "innocent" or "awful"
	similar to query to the one we started with...why compute scalar?
*/
SELECT COUNT(*)
FROM [Sales].[SalesOrderDetail];



/*
	Consider a Scalar UDF, such as one that counts how many different products each salesperson has made. 
	If you query Sales.SalesPerson, passing SalesPersonID into your UDF, 
	you will see your execution plan doesnt even touch Sales.SalesOrderHeader or Sales.SalesOrderDetail.
*/
CREATE FUNCTION dbo.CountProductsSold (
	@SalesPersonID INT
	) RETURNS INT

AS

BEGIN
    DECLARE @SoldCount INT;
	SELECT @SoldCount = COUNT(DISTINCT [d].[ProductID])
	FROM [Sales].[SalesOrderHeader] AS [h]
	JOIN [Sales].[SalesOrderDetail] AS [d] ON [d].[SalesOrderID] = [h].[SalesOrderID]
	WHERE [h].[SalesPersonID] = @SalesPersonID;
	
    RETURN (@SoldCount);

END

GO


/*
	What's the cost of the Compute Scalar?
*/
SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];


/*
	re-run with STATISTICS IO enabled
*/	
SET STATISTICS IO ON;
GO
SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];


/*
	How do you see what's going on?
	XE or Profiler
	**set session_id**
*/
DECLARE @SQLcmd NVARCHAR(MAX) = N'
CREATE EVENT SESSION [CaptureQueryIO] ON SERVER 
ADD EVENT sqlserver.sp_statement_completed(SET collect_statement=(1)
    WHERE ([sqlserver].[session_id]=(' + CAST(@@SPID AS NVARCHAR) + '))),
ADD EVENT sqlserver.sql_batch_completed(
    WHERE ([sqlserver].[session_id]=(' + CAST(@@SPID AS NVARCHAR) + '))) 
ADD TARGET package0.event_file(SET filename=N''C:\temp\CaptureQueryIO.xel'')
WITH (MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,MAX_DISPATCH_LATENCY=5 SECONDS,MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,TRACK_CAUSALITY=ON,STARTUP_STATE=OFF)'

PRINT @SQLCmd
EXECUTE(@SQLCmd)
GO

ALTER EVENT SESSION [CaptureQueryIO]
ON SERVER
STATE = START;
GO

SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];
GO

/*
	check XE data
*/
ALTER EVENT SESSION [CaptureQueryIO]
ON SERVER
STATE = STOP;

/*
	what do we see in the plan cache?
*/

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
SELECT	[qs].[creation_time],
		[qs].[last_execution_time],
		[qs].[execution_count],
		[qs].[total_logical_reads],
		[qs].[total_logical_reads]/[qs].[execution_count] AS [average_logical_reads],
		[dest].[text], 
		[plan].[query_plan]
FROM [sys].[dm_exec_query_stats] [qs]
CROSS APPLY [sys].[dm_exec_sql_text]([qs].[plan_handle]) [dest]
CROSS APPLY [sys].[dm_exec_query_plan]([qs].[plan_handle]) [plan]
WHERE [dest].[text] LIKE '%CountProductsSold%'
OPTION(MAXDOP 1, RECOMPILE);

/*
	run it again and recheck
*/
SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];
GO

/*
	what do we see in the plan cache?
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
SELECT	[qs].[creation_time],
		[qs].[last_execution_time],
		[qs].[execution_count],
		[qs].[total_logical_reads],
		[qs].[total_logical_reads]/[qs].[execution_count] AS [average_logical_reads],
		[dest].[text], 
		[plan].[query_plan]
FROM [sys].[dm_exec_query_stats] [qs]
CROSS APPLY [sys].[dm_exec_sql_text]([qs].[plan_handle]) [dest]
CROSS APPLY [sys].[dm_exec_query_plan]([qs].[plan_handle]) [plan]
WHERE [dest].[text] LIKE '%CountProductsSold%'
OPTION(MAXDOP 1, RECOMPILE);

/*
	Estimated Plan?
*/
SELECT 
	[BusinessEntityID] AS [SalesPerson], 
	[dbo].[CountProductsSold]([BusinessEntityID]) AS [NumberOfProductsSold]
FROM [Sales].[SalesPerson];

 /*
	clean up
 */
DROP FUNCTION dbo.CountProductsSold;
GO

DROP EVENT SESSION [CaptureQueryIO] 
	ON SERVER;
GO


/*
	End
*/